
namespace cpse {
namespace math {

template<int R, int C> template<class T> inline
matrix<R, C>::matrix(const tmatrix_dim<R, C, T> &m) {
	typedef typename assign_dispatch<matrix<R, C>, T>::handler handler;
	handler::go(*this, m);
}

template<int R, int C> template<class T> inline
matrix<R, C>::matrix(const tmatrix_ndim<T> &m) {
	typedef typename assign_dispatch<matrix<R, C>, T>::handler handler;
	handler::go(*this, m);
}

template<int R, int C> template<class T> inline
matrix<R, C> &matrix<R, C>::operator=(const tmatrix_dim<R, C, T> &m)
{
	typedef typename assign_dispatch<matrix<R, C>, T>::handler handler;
	handler::go(*this, m);
	return *this;
}

template<int R, int C> template<class T> inline
matrix<R, C> &matrix<R, C>::operator=(const tmatrix_ndim<T> &m)
{
	typedef typename assign_dispatch<matrix<R, C>, T>::handler handler;
	handler::go(*this, m);
	return *this;
}

template<int R> template<class T> inline
vector<R>::vector(const tmatrix_dim<R, 1, T> &v) {
	typedef typename assign_dispatch<vector<R>, T>::handler handler;
	handler::go(*this, v);
}

template<int R> template<class T> inline
vector<R>::vector(const tmatrix_ndim<T> &v) {
	typedef typename assign_dispatch<vector<R>, T>::handler handler;
	handler::go(*this, v);
}

template<int R> template<class T> inline
vector<R> &vector<R>::operator=(const tmatrix_dim<R, 1, T> &v) { 
	typedef typename assign_dispatch<vector<R>, T>::handler handler;
	handler::go(*this, v);
	return *this;
}

template<int R> template<class T> inline
vector<R> &vector<R>::operator=(const tmatrix_ndim<T> &v) {
	typedef typename assign_dispatch<vector<R>, T>::handler handler;
	handler::go(*this, v);
	return *this;
}

template<int R, int C> template<class T> inline
basic_matrix<R, C>::basic_matrix(const T &m) {
	typedef typename assign_dispatch<matrix<R, C>, T>::handler handler;
	handler::go(*this, m);
}

template<bool Element_op, class Base> inline
tmatrix_type<true, true, Element_op, Base>::operator float() const { 
	float_keeper t = *this;
	return t.f;
}
template<bool Element_op, class Base> inline
tmatrix_type<false, true, Element_op, Base>::operator float() const { 
	float_keeper t = *this;
	return t.f;
}

template<class T> float_keeper::float_keeper(const T &m) {
	typedef typename assign_dispatch<float_keeper, T>::handler handler;
	handler::go(*this, m);
}

} // namespace math
} // namespace cpse
